/*
 * This file is part of the Polkit-tqt project
 * Copyright (C) 2009 Jaroslav Reznik <jreznik@redhat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB. If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <TQtCore/TQDebug>
#include <TQInputDialog>

#include "klistener.h"
#include "agent/polkittqt1-agent-session.h"

using namespace PolkitTQt1::Agent;

KListener::KListener(TQObject *parent)
        : Listener(parent)
{
    tqDebug() << "Registering KDE listener";
}

// README: this is just testing code...

void KListener::initiateAuthentication(const TQString &actionId,
                                       const TQString &message,
                                       const TQString &iconName,
                                       const PolkitTQt1::Details &details,
                                       const TQString &cookie,
                                       const PolkitTQt1::Identity::List &identities,
                                       AsyncResult *result)
{
    tqDebug() << "initiateAuthentication for " << actionId << " with message " << message;
    tqDebug() << "iconName " << iconName;
    tqDebug() << details.keys();
    tqDebug() << "cookie" << cookie;

    TQ_FOREACH (const PolkitTQt1::Identity &identity, identities) {
        tqDebug() << identity.toString();
        Session *session;
        session = new Session(identity, cookie, result);
        connect(session, SIGNAL(request(TQString, bool)), this, SLOT(request(TQString, bool)));
        connect(session, SIGNAL(completed(bool)), this, SLOT(completed(bool)));
        connect(session, SIGNAL(showError(TQString)), this, SLOT(showError(TQString)));
        connect(session, SIGNAL(showInfo(TQString)), this, SLOT(showInfo(TQString)));
        session->initiate();
    }
}

bool KListener::initiateAuthenticationFinish()
{
    tqDebug() << "initiateAuthenticationFinish()";
    return true;
}

void KListener::cancelAuthentication()
{
    tqDebug() << "Cancelling authentication";
}

void KListener::request(const TQString &request, bool echo)
{
    tqDebug() << "Request: " << request;

    Session *session = (Session *)sender();

    session->setResponse("");
}

void KListener::completed(bool gainedAuthorization)
{
    tqDebug() << "Completed: " << gainedAuthorization;
    Session *session = (Session *)sender();

    session->result()->setCompleted();

    delete session;
}

void KListener::showError(const TQString &text)
{
    tqDebug() << "Error: " << text;
}

void KListener::showInfo(const TQString &text)
{
    tqDebug() << "Info: " << text;
}
